@@ -26,9 +26,10 @@
 #include "detail/ordered_block_id.hpp"
 
 #include "config_types.hpp"
-#include "device_scan_by_key_config.hpp"
+#include "detail/config/device_scan_by_key.hpp"
 
 #include "../config.hpp"
+#include "../detail/temp_storage.hpp"
 #include "../detail/various.hpp"
 #include "../functional.hpp"
 #include "../types/future_value.hpp"
@@ -132,31 +133,39 @@ namespace detail
 
         constexpr unsigned int block_size       = config::block_size;
         constexpr unsigned int items_per_thread = config::items_per_thread;
-        constexpr auto         items_per_block  = block_size * items_per_thread;
+        constexpr unsigned int items_per_block  = block_size * items_per_thread;
 
-        static constexpr size_t size_limit = config::size_limit;
-        static constexpr size_t aligned_size_limit
-            = ::rocprim::max<size_t>(size_limit - size_limit % items_per_block, items_per_block);
+        static constexpr unsigned int size_limit = config::size_limit;
+        static constexpr unsigned int aligned_size_limit
+            = std::max(size_limit - size_limit % items_per_block, items_per_block);
 
-        const size_t limited_size     = std::min<size_t>(size, aligned_size_limit);
-        const bool   use_limited_size = limited_size == aligned_size_limit;
+        const unsigned int limited_size
+            = static_cast<unsigned int>(std::min<size_t>(size, aligned_size_limit));
+        const bool use_limited_size = limited_size == aligned_size_limit;
 
         // Number of blocks in a single launch (or the only launch if it fits)
         const unsigned int number_of_blocks = ceiling_div(limited_size, items_per_block);
 
-        // Calculate required temporary storage, this is valid even with scan_state_with_sleep_type
-        const size_t scan_state_bytes
-            = align_size(scan_state_type::get_storage_size(number_of_blocks));
-        if(temporary_storage == nullptr)
+        void*                           scan_state_storage;
+        ordered_block_id_type::id_type* ordered_bid_storage;
+        wrapped_type*                   previous_last_value;
+
+        const hipError_t partition_result = detail::temp_storage::partition(
+            temporary_storage,
+            storage_size,
+            detail::temp_storage::make_linear_partition(
+                // This is valid even with offset_scan_state_with_sleep_type
+                detail::temp_storage::make_partition(
+                    &scan_state_storage,
+                    scan_state_type::get_temp_storage_layout(number_of_blocks)),
+                detail::temp_storage::make_partition(
+                    &ordered_bid_storage,
+                    ordered_block_id_type::get_temp_storage_layout()),
+                detail::temp_storage::ptr_aligned_array(&previous_last_value,
+                                                        use_limited_size ? 1 : 0)));
+        if(partition_result != hipSuccess || temporary_storage == nullptr)
         {
-            const size_t ordered_block_id_bytes
-                = align_size(ordered_block_id_type::get_storage_size(), alignof(wrapped_type));
-
-            // storage_size is never zero
-            storage_size = scan_state_bytes + ordered_block_id_bytes
-                           + (use_limited_size ? sizeof(wrapped_type) : 0);
-
-            return hipSuccess;
+            return partition_result;
         }
 
         if(number_of_blocks == 0u)
@@ -174,9 +183,11 @@ namespace detail
         // the value of use_sleep_scan_state
         auto with_scan_state
             = [use_sleep,
-               scan_state            = scan_state_type::create(temporary_storage, number_of_blocks),
-               scan_state_with_sleep = scan_state_with_sleep_type::create(
-                   temporary_storage, number_of_blocks)](auto&& func) mutable -> decltype(auto) {
+               scan_state = scan_state_type::create(scan_state_storage, number_of_blocks),
+               scan_state_with_sleep
+               = scan_state_with_sleep_type::create(scan_state_storage, number_of_blocks)](
+                  auto&& func) mutable -> decltype(auto)
+        {
             if(use_sleep)
             {
                 return func(scan_state_with_sleep);
@@ -188,15 +199,7 @@ namespace detail
         };
 
         // Create and initialize ordered_block_id obj
-        auto* const ptr         = static_cast<char*>(temporary_storage);
-        const auto  ordered_bid = ordered_block_id_type::create(
-             reinterpret_cast<ordered_block_id_type::id_type*>(ptr + scan_state_bytes));
-
-        // The last element
-        auto* const previous_last_value
-            = use_limited_size
-                  ? reinterpret_cast<wrapped_type*>(ptr + storage_size - sizeof(wrapped_type))
-                  : nullptr;
+        const auto ordered_bid = ordered_block_id_type::create(ordered_bid_storage);
 
         // Total number of blocks in all launches
         const auto   total_number_of_blocks = ceiling_div(size, items_per_block);
@@ -216,9 +219,13 @@ namespace detail
 
         for(size_t i = 0, offset = 0; i < number_of_launch; i++, offset += limited_size)
         {
-            const size_t current_size = std::min<size_t>(size - offset, limited_size);
-            const auto   scan_blocks  = ceiling_div(current_size, items_per_block);
-            const auto init_grid_size = ceiling_div(scan_blocks, block_size);
+            // limited_size is of type unsigned int, so current_size also fits in an unsigned int
+            // size_t is necessary as type of std::min because 'size - offset' can exceed the
+            // upper limit of unsigned int and converting it can lead to wrong results
+            const unsigned int current_size
+                = static_cast<unsigned int>(std::min<size_t>(size - offset, limited_size));
+            const unsigned int scan_blocks    = ceiling_div(current_size, items_per_block);
+            const unsigned int init_grid_size = ceiling_div(scan_blocks, block_size);
 
             // Start point for time measurements
             std::chrono::high_resolution_clock::time_point start;
@@ -250,25 +257,28 @@ namespace detail
             {
                 start = std::chrono::high_resolution_clock::now();
             }
-            with_scan_state([&](auto& scan_state) {
-                hipLaunchKernelGGL(HIP_KERNEL_NAME(device_scan_by_key_kernel<Exclusive, config>),
-                                   dim3(scan_blocks),
-                                   dim3(block_size),
-                                   0,
-                                   stream,
-                                   keys + offset,
-                                   input + offset,
-                                   output + offset,
-                                   initial_value,
-                                   compare,
-                                   scan_op,
-                                   scan_state,
-                                   size,
-                                   i * number_of_blocks,
-                                   total_number_of_blocks,
-                                   ordered_bid,
-                                   i > 0 ? previous_last_value : nullptr);
-            });
+            with_scan_state(
+                [&](auto& scan_state)
+                {
+                    hipLaunchKernelGGL(
+                        HIP_KERNEL_NAME(device_scan_by_key_kernel<Exclusive, config>),
+                        dim3(scan_blocks),
+                        dim3(block_size),
+                        0,
+                        stream,
+                        keys + offset,
+                        input + offset,
+                        output + offset,
+                        initial_value,
+                        compare,
+                        scan_op,
+                        scan_state,
+                        size,
+                        i * number_of_blocks,
+                        total_number_of_blocks,
+                        ordered_bid,
+                        i > 0 ? as_const_ptr(previous_last_value) : nullptr);
+                });
             ROCPRIM_DETAIL_HIP_SYNC_AND_RETURN_ON_ERROR(
                 "device_scan_by_key_kernel", current_size, start);
         }
@@ -368,7 +378,7 @@ namespace detail
 ///     values_output, size,
 ///     rocprim::plus<int>()
 /// );
-/// // values_output: [1, 2, 3, 7, 5, 11, 18, 8]
+/// // values_output: [1, 3, 3, 7, 5, 11, 18, 8]
 /// \endcode
 /// \endparblock
 template <typename Config = default_config,
